/* ----------------------------------------------------------------------
* I-SIMPA (http://i-simpa.ifsttar.fr). This file is part of I-SIMPA.
*
* I-SIMPA is a GUI for 3D numerical sound propagation modelling dedicated
* to scientific acoustic simulations.
* Copyright (C) 2007-2014 - IFSTTAR - Judicael Picaut, Nicolas Fortin
*
* I-SIMPA is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 3 of the License, or
* (at your option) any later version.
* 
* I-SIMPA is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software Foundation,
* Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA or 
* see <http://ww.gnu.org/licenses/>
*
* For more information, please consult: <http://i-simpa.ifsttar.fr> or 
* send an email to i-simpa@ifsttar.fr
*
* To contact Ifsttar, write to Ifsttar, 14-20 Boulevard Newton
* Cite Descartes, Champs sur Marne F-77447 Marne la Vallee Cedex 2 FRANCE
* or write to scientific.computing@ifsttar.fr
* ----------------------------------------------------------------------*/


/*! \page addElement Ajout d'un lment au sein du projet
 * 
 * Voici la marche  suivre pour ajouter un nouveau type de donnes de projet dans un des trois arbres (scene,core ou resultats)
 *
 * \section addElementstepone Crer le fichier d'entte
 * Il faut crer le fichier vide ".h" dans le dossier correspondant  l'arbre o l'lment sera prsent :
 *	- data_manager\\tree_scene\\ Pour les lments du premier onglet
 *	- data_manager\\tree_core\\ Pour les lments du deuxime onglet
 *	- data_manager\\tree_rapport\\ Pour les lments du troisime onglet
 *
 * Le nom du fichier doit donner une indication sur sa place dans la hirarchie, il suffit de se rfrer aux noms des autres lments.
 *
 * \section addElementsteptwo Constructeurs de base d'un lment
 * Il faut directement hriter de Element pour un lment n'ayant pas de comportement spcial (un dossier ou un lment de proprit)
 *
 * \include demos/demos_add_element.h
 * Le premier constructeur est le constructeur XML. Il est appel lors du chargement d'un projet existant.
 *
 * Le deuxime constructeur est le constructeur de cration. Il est appel lors de la cration d'un noueau projet ou en cours d'execution du programme.
 * La mthode SaveXMLDoc() est la surcharge permettant de sauvegarder les donnes propres  l'lment pour une rcuperation ultrieure dans le constructeur XML.
 *
 * @warning Il faut donner un indice  ce nouvel lment en rajoutant une ligne  la fin de l'numeration des lments prsent dans element.h, voir Element::ELEMENT_TYPE .
 *
 * \section addElementstepthree Instanciation de la classe
 * Cette classe n'est pour l'instant pas utilis. Pour qu'elle le soit il faut ajouter le code d'instanciation de notre nouvelle classe.
 * Pour cela il faut ouvrir le fichier d'entete de l'lment parent.
 * Voici un exemple afin de crer toujours l'lment demo  la cration de E_Scene_Projet et charger le ou les lment Demo au chargement d'un projet.
 * \include demos/demos_add_element_parent.h
 * Comme vous pouvez le voir il y a 3 modifications :
 *	- Ajout du fichier d'entete de la classe 
 *	  \skipline e_scene_projet_demo
 *	- Les deux lignes de test lors du chargement du fichier XML (Test d'galit des indices)
 *	  \skip MARKERBEGIN
 *	  \until MARKEREND
 *	- La ligne de cration dans le deuxime constructeur.
 *	  \skipline MARKERAPPEND
 *
 * Compilez, executez le projet et cliquez sur "Nouveau projet". Vous pouvez voir votre lment dans l'arbre du projet. Cependant l'intraction avec cet lment est trs limite.
 *
 * \section addElementstepfour Ajout de proprits (facultatif)
 * La classe Element dispose de plusieurs mthodes afin de contenir des proprits tel qu'un champ texte et un champ de couleur (@ref addprop "Ensemble de mthodes"):
 * Reprenons notre exemple. Nous allons ajouter un champ de texte et un champ de couleur.
 * 
 * \include demos/demos_add_element_withprop.h
 * 
 * On a seulement ajout ces lignes :
 * \skip MARKERBEGIN
 * \until MARKEREND
 *
 * Vous vous demandez peut tre quel est l'utilit des lignes comportant uniquement @ref traductionTuto "la mthode de traduction" ?
 *
 * En fait la raison est simple. Le libell que vous renseignez dans l'appel de la mthode d'ajout de proprit sera sauvegard dans le fichier de projet comme tant une chaine non traduite.
 * Si jamais vous utilisez la mthode de traduction _( ) dans ses paramtres alors les libell des proprits d'un projet qui sera chang entre un anglais et un francais resteront en anglais.
 * Donc utiliser la mthode de traduction en dehors des proprits permet au champ d'tre traduit uniquement lors de l'affichage du libell et que cette chaine  traduire soit reconnue par le programme de mise  jour des chaines  traduire.
 *
 * Vous pouvez ensuite utiliser \ref getprop "les accesseurs de proprits" pour lire ses donnes. Consultez \ref gestionpropelement "la documentation relative" pour plus de dtails.
 *
 * \section addElementstepfive Rendu 3D de l'lment
 *
 * Nous allons modifier notre lment de dmonstration pour le dessiner sur la vue 3D.
 * \include demos/demos_add_element_drawable.h
 *
 * En premier lieu il ne faut plus inclure directement element.h mais drawable_element.h
 * \skipline drawable_element
 *
 * Nous n'allons plus hriter directement de Element. On va remplacer la classe d'hritage par E_Drawable, cette classe permet justement le rendu 3D d'un lment.
 * \skipline MARKER0
 * Il en va de mme pour les deux constructeurs.
 *
 * Le constructeur XML
 * \skip MARKER1
 * \until {
 * Le constructeur de cration
 * \skip MARKER2
 * \until {
 * 
 * La modification consiste galement  la surcharge de la mthode E_Drawable::DrawItem() afin de commander les mthodes de dessins adaptes  notre nouvel lment.
 * \skip DrawItem
 * \until }
 * 
 * Le libell de l'lment est ds  prsent visible dans la vue 3D. Les modifications des proprits sont visibles immdiatement dans la vue 3D.
 */
